@@ -34,6 +34,8 @@
 #include <fcntl.h>
 #include <assert.h>
 #include <errno.h>
+#include <netdb.h>
+#include <arpa/inet.h>
 
 #include <gcrypt.h>
 #include <gnutls/gnutls.h>
@@ -90,6 +92,7 @@
 
 struct prelude_client {
 
+        int refcount;
         int flags;
         int status;
 
@@ -211,6 +214,8 @@ static void gen_heartbeat(prelude_client_t *client)
         idmef_message_t *message;
         idmef_heartbeat_t *heartbeat;
 
+        prelude_log_debug(2, "running heartbeat callback.\n");
+
         ret = idmef_message_new(&message);
         if ( ret < 0 ) {
                 prelude_perror(ret, "error creating new IDMEF message");
@@ -277,6 +282,7 @@ static void setup_heartbeat_timer(prelude_client_t *client, int expire)
 }
 
 
+#ifdef HAVE_IPV6
 static prelude_bool_t is_loopback_ipv6(struct in6_addr *addr)
 {
         struct in6_addr lo;
@@ -285,6 +291,7 @@ static prelude_bool_t is_loopback_ipv6(struct in6_addr *addr)
 
         return (memcmp(addr, &lo, sizeof(lo)) == 0) ? TRUE : FALSE;
 }
+#endif
 
 
 static prelude_bool_t is_loopback_ipv4(struct in_addr *addr)
@@ -298,8 +305,10 @@ static prelude_bool_t is_loopback(int family, void *addr)
         if ( family == AF_INET )
                 return is_loopback_ipv4(addr);
 
+#ifdef HAVE_IPV6
         else if ( family == AF_INET6 )
                 return is_loopback_ipv6(addr);
+#endif
 
         else
                 return FALSE;
@@ -313,6 +322,9 @@ static int set_analyzer_host_info(idmef_analyzer_t *analyzer, const char *node_s
         idmef_address_t *addr;
         prelude_string_t *str;
 
+        if ( ! node_str && ! addr_str )
+                return 0;
+
         ret = idmef_analyzer_new_node(analyzer, &node);
         if ( ret < 0 )
                 return ret;
@@ -321,22 +333,24 @@ static int set_analyzer_host_info(idmef_analyzer_t *analyzer, const char *node_s
         if ( ret < 0 )
                 return ret;
 
-        if ( prelude_string_is_empty(str) )
+        if ( node_str && prelude_string_is_empty(str) )
                 prelude_string_set_dup(str, node_str);
 
-        if ( ! (addr = idmef_node_get_next_address(node, NULL)) ) {
-                ret = idmef_node_new_address(node, &addr, 0);
+        if ( addr_str ) {
+                if ( ! (addr = idmef_node_get_next_address(node, NULL)) ) {
+                        ret = idmef_node_new_address(node, &addr, 0);
+                        if ( ret < 0 )
+                                return ret;
+                }
+
+                ret = idmef_address_new_address(addr, &str);
                 if ( ret < 0 )
                         return ret;
+
+                if ( prelude_string_is_empty(str) )
+                        prelude_string_set_dup(str, addr_str);
         }
 
-        ret = idmef_address_new_address(addr, &str);
-        if ( ret < 0 )
-                return ret;
-
-        if ( prelude_string_is_empty(str) )
-                prelude_string_set_dup(str, addr_str);
-
         return 0;
 }
 
@@ -1146,6 +1160,7 @@ int prelude_client_new(prelude_client_t **client, const char *profile)
         gl_lock_init(new->msgbuf_lock);
         prelude_timer_init_list(&new->heartbeat_timer);
 
+        new->refcount = 1;
         new->flags = PRELUDE_CLIENT_FLAGS_HEARTBEAT|PRELUDE_CLIENT_FLAGS_CONNECT|PRELUDE_CLIENT_FLAGS_AUTOCONFIG;
         new->permission = PRELUDE_CONNECTION_PERMISSION_IDMEF_WRITE;
 
@@ -1197,6 +1212,16 @@ int prelude_client_new(prelude_client_t **client, const char *profile)
 
 
 
+prelude_client_t *prelude_client_ref(prelude_client_t *client)
+{
+        prelude_return_val_if_fail(client, NULL);
+
+        client->refcount++;
+        return client;
+}
+
+
+
 /**
  * prelude_client_init:
  * @client: Pointer to a #prelude_client_t object to initialize.
@@ -1552,6 +1577,9 @@ void prelude_client_destroy(prelude_client_t *client, prelude_client_exit_status
 {
         prelude_return_if_fail(client);
 
+        if ( --client->refcount )
+                return;
+
         prelude_timer_destroy(&client->heartbeat_timer);
 
         if ( client->status >= CLIENT_STATUS_STARTING     &&
