@@ -28,24 +28,28 @@
 #include <stdlib.h>
 #include <unistd.h>
 #include <gnutls/gnutls.h>
+#include <gcrypt.h>
+
+#include "glthread/thread.h"
+#include "glthread/lock.h"
 
 #include "prelude.h"
 #include "idmef-path.h"
 #include "prelude-option.h"
 #include "prelude-log.h"
 #include "prelude-timer.h"
-#include "prelude-thread.h"
 #include "variable.h"
 #include "tls-auth.h"
 
 
 int _prelude_internal_argc = 0;
+char *_prelude_prefix = NULL;
 char *_prelude_internal_argv[1024];
 
 char _prelude_init_cwd[PATH_MAX];
 static int libprelude_refcount = 0;
 extern prelude_option_t *_prelude_generic_optlist;
-extern pthread_mutex_t _criteria_parse_mutex;
+extern gl_lock_t _criteria_parse_mutex;
 
 
 
@@ -54,31 +58,51 @@ static void tls_log_func(int level, const char *data)
         prelude_log(PRELUDE_LOG_INFO, "%s", data);
 }
 
-
-
-static void prepare_fork_cb(void)
+static int gcry_prelude_mutex_init(void **retval)
 {
-        _idmef_path_cache_lock();
-        prelude_thread_mutex_lock(&_criteria_parse_mutex);
+        int ret;
+        gl_lock_t *lock;
+
+        *retval = lock = malloc(sizeof(*lock));
+        if ( ! lock )
+                return ENOMEM;
+
+        ret = glthread_lock_init(lock);
+        if ( ret < 0 )
+                free(lock);
+
+        return ret;
+}
+
+
+static int gcry_prelude_mutex_destroy(void **lock)
+{
+        return glthread_lock_destroy(*lock);
 }
 
 
 
-static void parent_fork_cb(void)
+static int gcry_prelude_mutex_lock(void **lock)
 {
-        _idmef_path_cache_unlock();
-        prelude_thread_mutex_unlock(&_criteria_parse_mutex);
+        return glthread_lock_lock((gl_lock_t *) *lock);
 }
 
 
-
-static void child_fork_cb(void)
+static int gcry_prelude_mutex_unlock(void **lock)
 {
-        _idmef_path_cache_reinit();
-        prelude_thread_mutex_init(&_criteria_parse_mutex, NULL);
+        return glthread_lock_unlock((gl_lock_t *) *lock);
 }
 
 
+static struct gcry_thread_cbs gcry_threads_prelude = {
+        GCRY_THREAD_OPTION_USER,
+        NULL,
+        gcry_prelude_mutex_init,
+        gcry_prelude_mutex_destroy,
+        gcry_prelude_mutex_lock,
+        gcry_prelude_mutex_unlock
+};
+
 
 static void slice_arguments(int *argc, char **argv)
 {
@@ -133,7 +157,6 @@ static void slice_arguments(int *argc, char **argv)
 }
 
 
-
 /**
  * prelude_init:
  * @argc: Address of the argc parameter of your main() function.
@@ -168,6 +191,10 @@ int prelude_init(int *argc, char **argv)
         if ( env )
                 prelude_log_set_logfile(env);
 
+        env = getenv("LIBPRELUDE_PREFIX");
+        if ( env )
+                _prelude_prefix = strdup(env);
+
         env = getenv("LIBPRELUDE_ABORT");
         if ( env ) {
                 if ( *env )
@@ -176,7 +203,7 @@ int prelude_init(int *argc, char **argv)
                         _prelude_log_set_abort_level(PRELUDE_LOG_CRIT);
         }
 
-        _prelude_thread_in_use();
+        prelude_thread_init(NULL);
 
         if ( ! getcwd(_prelude_init_cwd, sizeof(_prelude_init_cwd)) )
                 _prelude_init_cwd[0] = 0;
@@ -185,12 +212,22 @@ int prelude_init(int *argc, char **argv)
         if ( ret < 0 )
                 return ret;
 
-        ret = prelude_thread_atfork(prepare_fork_cb, parent_fork_cb, child_fork_cb);
+        ret = glthread_atfork(prelude_fork_prepare, prelude_fork_parent, prelude_fork_child);
         if ( ret != 0 )
                 return prelude_error_from_errno(ret);
 
         slice_arguments(argc, argv);
 
+        ret = gcry_control(GCRYCTL_SET_THREAD_CBS, &gcry_threads_prelude);
+        if ( ret < 0 )
+                return prelude_error_verbose(PRELUDE_ERROR_TLS,
+                                             "gcrypt initialization failed: %s", gcry_strerror(ret));
+
+        ret = gnutls_global_init();
+        if ( ret < 0 )
+                return prelude_error_verbose(PRELUDE_ERROR_TLS,
+                                             "TLS initialization failed: %s", gnutls_strerror(ret));
+
         return 0;
 }
 
@@ -262,3 +299,46 @@ const char *prelude_check_version(const char *req_version)
 
         return NULL;
 }
+
+
+
+void prelude_fork_prepare(void)
+{
+#ifdef HAVE_PTHREAD_ATFORK
+        return;
+#endif
+
+        _prelude_async_fork_prepare();
+        _prelude_timer_fork_prepare();
+
+        _idmef_path_cache_lock();
+        gl_lock_lock(_criteria_parse_mutex);
+}
+
+
+void prelude_fork_parent(void)
+{
+#ifdef HAVE_PTHREAD_ATFORK
+        return;
+#endif
+
+        _prelude_async_fork_parent();
+        _prelude_timer_fork_parent();
+
+        _idmef_path_cache_unlock();
+        gl_lock_unlock(_criteria_parse_mutex);
+}
+
+
+void prelude_fork_child(void)
+{
+#ifdef HAVE_PTHREAD_ATFORK
+        return;
+#endif
+
+        _prelude_async_fork_child();
+        _prelude_timer_fork_child();
+
+        _idmef_path_cache_reinit();
+        gl_lock_init(_criteria_parse_mutex);
+}
